#include <stdio.h>           // Standard C/C++ Input-Output
#include <math.h>            // Math Functions
#include <windows.h>         // Standard Header For MSWindows Applications
#include <gl/glut.h>            // The GL Utility Toolkit (GLUT) Header
#include <time.h>

struct vector3d
{
	float X, Y, Z;

	inline vector3d(void) {}
	inline vector3d(const float x, const float y, const float z)
	{
		X = x; Y = y; Z = z;
	}

	inline vector3d operator + (const vector3d& A) const
	{
		return vector3d(X + A.X, Y + A.Y, Z + A.Z);
	}

	inline vector3d operator + (const float A) const
	{
		return vector3d(X + A, Y + A, Z + A);
	}

	inline float Dot(const vector3d& A) const
	{
		return A.X * X + A.Y * Y + A.Z * Z;
	}
};

// The Following Directive Fixes The Problem With Extra Console Window
//#pragma comment(linker, "/subsystem:\"windows\" /entry:\"mainCRTStartup\"")

// Uncomment the following if you want to use the glut library from the current directory
//#pragma comment(lib, "lib/glut32.lib")

// Global Variables
bool g_gamemode;				// GLUT GameMode ON/OFF
bool g_fullscreen;				// Fullscreen Mode ON/OFF (When g_gamemode Is OFF)
bool b_culling = false;			// Culling Enabled is Mandatory for this assignment do not change
float aspect = 1;

const float PI = 3.1415926535897932384626433832795028;
const float epsilon = 0.001;




//Function Prototypes
void render(void);
void initLights(void);
bool init(void);
void reshape(int w,int h);
void keyboard(unsigned char key, int x, int y);
void special_keys(int a_keys, int x, int y);
void drawAxes();


void initLights(void) {

	glEnable(GL_LIGHT0);							   // Quick And Dirty Lighting (Assumes Light0 Is Set Up)
	glEnable(GL_LIGHTING);							   // Enable Lighting
}


// Our GL Specific Initializations. Returns true On Success, false On Fail.
bool init(void)
{
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);             // Pixel Storage Mode To Byte Alignment
	glEnable(GL_TEXTURE_2D);                           // Enable Texture Mapping 
    glClearColor(0.0f, 0.0f, 0.0f, 0.5f);			   // Black Background (CHANGED)
    glClearDepth(1.0f);								   // Depth Buffer Setup
    glDepthFunc(GL_LEQUAL);							   // The Type Of Depth Testing To Do
    glEnable(GL_DEPTH_TEST);						   // Enables Depth Testing
    glShadeModel(GL_SMOOTH);						   // Enable Smooth Shading
	initLights();
	glEnable(GL_COLOR_MATERIAL);					   // Enable Material Coloring
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Hint for nice perspective interpolation
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);	// Set the color tracking for both faces for both the ambient and diffuse components
	glEnable(GL_NORMALIZE);
	glFrontFace(GL_CCW);                               //Counter Clock Wise definition of the front and back side of faces
	glCullFace(GL_BACK);                               //Hide the back side
	

	return true;
}

void positionCamera(){

	glMatrixMode(GL_PROJECTION);     // Select The Projection Matrix

	glLoadIdentity();                // Reset The Projection Matrix
	gluPerspective(45.0f, aspect, 0.1, 100.0);
	gluLookAt(0.0f,0.0f,30.0f,0.0f,1.0f,0.0f,0.0f,1.0f,0.0f);
	
	//camera transformations go here

	glMatrixMode(GL_MODELVIEW);      // Select The Modelview Matrix

}

// Our Reshaping Handler (Required Even In Fullscreen-Only Modes)
void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	// Calculate The Aspect Ratio And Set The Clipping Volume
	if (h == 0) h = 1;
	aspect = (float)w/(float)h;
	positionCamera();
	glMatrixMode(GL_MODELVIEW);      // Select The Modelview Matrix
	glLoadIdentity(); // Reset The Modelview Matrix
	initLights();
}

// Our Keyboard Handler (Normal Keys)
void keyboard(unsigned char key, int x, int y)
{
	switch (key) {
		case 'w':
			break;
		case 's':
			break;

		case 27:        // When Escape Is Pressed...
			exit(0);    // Exit The Program
			break;      // Ready For Next Case
		default:
		break;
	}
	glutPostRedisplay();
}

// Our Keyboard Handler For Special Keys (Like Arrow Keys And Function Keys)
void special_keys(int a_keys, int x, int y)
{

	switch (a_keys) {
		case GLUT_KEY_F1:
			// We Can Switch Between Windowed Mode And Fullscreen Mode Only
			if (!g_gamemode) {
				g_fullscreen = !g_fullscreen;       // Toggle g_fullscreen Flag
				if (g_fullscreen) glutFullScreen(); // We Went In Fullscreen Mode
				else glutReshapeWindow(500, 500);   // We Went In Windowed Mode
			}
		break;
		case GLUT_KEY_UP:

			break;
		case GLUT_KEY_DOWN:

			break;
		case GLUT_KEY_LEFT:
			break;
		case GLUT_KEY_RIGHT:
			break;
		default:
			;
	}

	glutPostRedisplay();
}



// Main Function For Bringing It All Together.
int main(int argc, char** argv)
{
	glutInit(&argc, argv);                           // GLUT Initializtion
	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGBA | GLUT_DOUBLE); // (CHANGED)|
	if (g_gamemode) {
		glutGameModeString("1024x768:32");            // Select 1024x768 In 32bpp Mode
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
			glutEnterGameMode();                     // Enter Full Screen
		else
			g_gamemode = false;                     // Cannot Enter Game Mode, Switch To Windowed
	}
	if (!g_gamemode) {
		glutInitWindowPosition(100, 100);
		glutInitWindowSize(500, 500);                  // Window Size If We Start In Windowed Mode
		glutCreateWindow("EPL426"); // Window Title 
	}
	if (!init()) {                                   // Our Initialization
		fprintf(stderr,"Initialization failed.");
		return -1;
	}
	glutDisplayFunc(render);                     // Register The Display Function
	glutReshapeFunc(reshape);                    // Register The Reshape Handler
	glutKeyboardFunc(keyboard);                  // Register The Keyboard Handler
	glutSpecialFunc(special_keys);               // Register Special Keys Handler
	glutIdleFunc(NULL);                        	 // We Do Rendering In Idle Time
	glutMainLoop();                              // Go To GLUT Main Loop
	return 0;
}

void RenderSquare(vector3d center, float size)
{
	float halfSize = size/2;

	glBegin(GL_TRIANGLES);
		//triangle 1
		glColor3f(0.0f, 1.0f, 0.0f);
		glVertex3f(center.X + halfSize, center.Y + halfSize, center.Z);
		glVertex3f(center.X + halfSize, center.Y - halfSize, center.Z);
		glVertex3f(center.X - halfSize, center.Y + halfSize, center.Z);

		//triangle 2
		glColor3f(1.0f, 1.0f, 0.0f);
		glVertex3f(center.X - halfSize, center.Y + halfSize, center.Z);
		glVertex3f(center.X - halfSize, center.Y - halfSize, center.Z);
		glVertex3f(center.X + halfSize, center.Y - halfSize, center.Z);
	glEnd();
}

void RenderSquare2(vector3d point1, vector3d point2, vector3d point3, vector3d point4, vector3d color)
{
	glColor3f(color.X, color.Y, color.Z);
	glBegin(GL_TRIANGLES);
		//triangle 1
		glVertex3f(point1.X, point1.Y, point1.Z);
		glVertex3f(point2.X, point2.Y, point2.Z);
		glVertex3f(point3.X, point3.Y, point3.Z);

		//triangle 2
		glVertex3f(point2.X, point2.Y, point2.Z);
		glVertex3f(point3.X, point3.Y, point3.Z);
		glVertex3f(point4.X, point4.Y, point4.Z);
	glEnd();
}

void RenderCube(vector3d center, float size)
{
	float halfSize = size / 2;

	// Front
	vector3d frontLeftUp = center + vector3d(-halfSize, halfSize, halfSize);
	vector3d frontRightUp = center + vector3d(halfSize, halfSize, halfSize);
	vector3d frontLeftDown = center + vector3d(-halfSize, -halfSize, halfSize);
	vector3d frontRightDown = center + vector3d(halfSize, -halfSize, halfSize);

	// Back
	vector3d backLeftUp = center + vector3d(-halfSize, halfSize, -halfSize);
	vector3d backRightUp = center + vector3d(halfSize, halfSize, -halfSize);
	vector3d backLeftDown = center + vector3d(-halfSize, -halfSize, -halfSize);
	vector3d backRightDown = center + vector3d(halfSize, -halfSize, -halfSize);

	// Front
	RenderSquare2(frontLeftUp, frontRightUp, frontLeftDown, frontRightDown, vector3d(1,0,0));
	// Right
	RenderSquare2(frontRightUp, frontRightDown, backRightUp, backRightDown, vector3d(1, 1, 0));
	// Back
	RenderSquare2(backLeftUp, backRightUp, backLeftDown, backRightDown, vector3d(0, 1, 0));
	// Left
	RenderSquare2(frontLeftUp, frontLeftDown, backLeftUp, backLeftDown, vector3d(0, 1, 1));
	// Up
	RenderSquare2(frontLeftUp, frontRightUp, backLeftUp, backRightUp, vector3d(0, 0, 1));
	// Down 
	RenderSquare2(frontLeftDown, frontRightDown, backLeftDown, backRightDown, vector3d(1, 1, 1));
}

// Our Rendering Is Done Here
void render(void)   
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer

	// Do we have culling enabled?
	if (b_culling == true)
		glEnable(GL_CULL_FACE);
	else
		glDisable(GL_CULL_FACE);


	glLoadIdentity();

	glColor3f(0.0f, 1.0f, 0.0f);

	glBegin(GL_TRIANGLES);
		//triangle 1
		glVertex3f(0, 0, 0);
		glVertex3f(0, 1, 0);
		glVertex3f(-1, 0, 0);
		//triangle 2
		glVertex3f(0, 0, 0);
		glVertex3f(0, -1, 0);
		glVertex3f(-1, 0, 0);
	glEnd();

	/*
	float sizeX = 1.0f;
	float sizeY = 1.0f;

	glBegin(GL_TRIANGLES);
		//triangle 1
		glVertex3f(sizeX, sizeY,0);
		glVertex3f(0, sizeY,0);
		glVertex3f(sizeX,0,0);

		glColor3f(1.0f, 1.0f, 0.0f);
		//triangle 2
		glVertex3f(0, sizeY,0);
		glVertex3f(0,0,0);
		glVertex3f(sizeX,0,0);
	glEnd();
	*/

	//RenderSquare(vector3d(0, 0, 0), 2);
	//RenderCube(vector3d(0, 0, 0), 2);

    // Swap The Buffers To Make Our Rendering Visible
    glutSwapBuffers();
	glutPostRedisplay(); //animation
}